from ._visible import VisibleValidator
from ._uid import UidValidator
from ._tickfont import TickfontValidator
from ._stream import StreamValidator
from ._showlegend import ShowlegendValidator
from ._selectedpoints import SelectedpointsValidator
from ._rangefont import RangefontValidator
from ._opacity import OpacityValidator
from ._name import NameValidator
from ._line import LineValidator
from ._legendgroup import LegendgroupValidator
from ._labelfont import LabelfontValidator
from ._idssrc import IdssrcValidator
from ._ids import IdsValidator
from ._hoverlabel import HoverlabelValidator
from ._hoverinfosrc import HoverinfosrcValidator
from ._hoverinfo import HoverinfoValidator
from ._domain import DomainValidator
from ._dimensions import DimensionsValidator
from ._customdatasrc import CustomdatasrcValidator
from ._customdata import CustomdataValidator



from ._width import WidthValidator
from ._smoothing import SmoothingValidator
from ._simplify import SimplifyValidator
from ._shape import ShapeValidator
from ._dash import DashValidator
from ._color import ColorValidator



import _plotly_utils.basevalidators


class TicksValidator(_plotly_utils.basevalidators.EnumeratedValidator):

    def __init__(
        self,
        plotly_name='ticks',
        parent_name='scatterpolar.marker.colorbar',
        **kwargs
    ):
        super(TicksValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop('edit_type', 'colorbars'),
            role=kwargs.pop('role', 'style'),
            values=kwargs.pop('values', ['outside', 'inside', '']),
            **kwargs
        )



import _plotly_utils.basevalidators


class XValidator(_plotly_utils.basevalidators.CompoundValidator):

    def __init__(
        self, plotly_name='x', parent_name='scatter3d.projection', **kwargs
    ):
        super(XValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop('data_class_str', 'X'),
            data_docs=kwargs.pop(
                'data_docs', """
            opacity
                Sets the projection color.
            scale
                Sets the scale factor determining the size of
                the projection marker points.
            show
                Sets whether or not projections are shown along
                the x axis.
"""
            ),
            **kwargs
        )



import _plotly_utils.basevalidators


class LineValidator(_plotly_utils.basevalidators.CompoundValidator):

    def __init__(self, plotly_name='line', parent_name='box', **kwargs):
        super(LineValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop('data_class_str', 'Line'),
            data_docs=kwargs.pop(
                'data_docs', """
            color
                Sets the color of line bounding the box(es).
            width
                Sets the width (in px) of line bounding the
                box(es).
"""
            ),
            **kwargs
        )



from ._zerolinewidth import ZerolinewidthValidator
from ._zerolinecolor import ZerolinecolorValidator
from ._zeroline import ZerolineValidator
from ._visible import VisibleValidator
from ._type import TypeValidator
from ._titlefont import TitlefontValidator
from ._title import TitleValidator
from ._tickwidth import TickwidthValidator
from ._tickvalssrc import TickvalssrcValidator
from ._tickvals import TickvalsValidator
from ._ticktextsrc import TicktextsrcValidator
from ._ticktext import TicktextValidator
from ._ticksuffix import TicksuffixValidator
from ._ticks import TicksValidator
from ._tickprefix import TickprefixValidator
from ._tickmode import TickmodeValidator
from ._ticklen import TicklenValidator
from ._tickformatstops import TickformatstopsValidator
from ._tickformat import TickformatValidator
from ._tickfont import TickfontValidator
from ._tickcolor import TickcolorValidator
from ._tickangle import TickangleValidator
from ._tick0 import Tick0Validator
from ._spikethickness import SpikethicknessValidator
from ._spikesnap import SpikesnapValidator
from ._spikemode import SpikemodeValidator
from ._spikedash import SpikedashValidator
from ._spikecolor import SpikecolorValidator
from ._side import SideValidator
from ._showticksuffix import ShowticksuffixValidator
from ._showtickprefix import ShowtickprefixValidator
from ._showticklabels import ShowticklabelsValidator
from ._showspikes import ShowspikesValidator
from ._showline import ShowlineValidator
from ._showgrid import ShowgridValidator
from ._showexponent import ShowexponentValidator
from ._separatethousands import SeparatethousandsValidator
from ._scaleratio import ScaleratioValidator
from ._scaleanchor import ScaleanchorValidator
from ._rangeslider import RangesliderValidator
from ._rangeselector import RangeselectorValidator
from ._rangemode import RangemodeValidator
from ._range import RangeValidator
from ._position import PositionValidator
from ._overlaying import OverlayingValidator
from ._nticks import NticksValidator
from ._mirror import MirrorValidator
from ._linewidth import LinewidthValidator
from ._linecolor import LinecolorValidator
from ._layer import LayerValidator
from ._hoverformat import HoverformatValidator
from ._gridwidth import GridwidthValidator
from ._gridcolor import GridcolorValidator
from ._fixedrange import FixedrangeValidator
from ._exponentformat import ExponentformatValidator
from ._dtick import DtickValidator
from ._domain import DomainValidator
from ._constraintoward import ConstraintowardValidator
from ._constrain import ConstrainValidator
from ._color import ColorValidator
from ._categoryorder import CategoryorderValidator
from ._categoryarraysrc import CategoryarraysrcValidator
from ._categoryarray import CategoryarrayValidator
from ._calendar import CalendarValidator
from ._autorange import AutorangeValidator
from ._automargin import AutomarginValidator
from ._anchor import AnchorValidator



import _plotly_utils.basevalidators


class UpValidator(_plotly_utils.basevalidators.CompoundValidator):

    def __init__(
        self, plotly_name='up', parent_name='layout.scene.camera', **kwargs
    ):
        super(UpValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop('data_class_str', 'Up'),
            data_docs=kwargs.pop(
                'data_docs', """
            x

            y

            z

"""
            ),
            **kwargs
        )



import os

# file structure
PLOTLY_DIR = os.environ.get("PLOTLY_DIR",
                            os.path.join(os.path.expanduser("~"), ".plotly"))

CREDENTIALS_FILE = os.path.join(PLOTLY_DIR, ".credentials")
CONFIG_FILE = os.path.join(PLOTLY_DIR, ".config")
TEST_FILE = os.path.join(PLOTLY_DIR, ".permission_test")

# this sets both the DEFAULTS and the TYPES for these files
FILE_CONTENT = {CREDENTIALS_FILE: {'username': '',
                                   'api_key': '',
                                   'proxy_username': '',
                                   'proxy_password': '',
                                   'stream_ids': []},
                CONFIG_FILE: {'plotly_domain': 'https://plot.ly',
                              'plotly_streaming_domain': 'stream.plot.ly',
                              'plotly_api_domain': 'https://api.plot.ly',
                              'plotly_ssl_verification': True,
                              'plotly_proxy_authorization': False,
                              'world_readable': True,
                              'sharing': 'public',
                              'auto_open': True}}


def _permissions():
    try:
        if not os.path.exists(PLOTLY_DIR):
            os.mkdir(PLOTLY_DIR)
        with open(TEST_FILE, 'w') as f:
            f.write('testing\n')
        os.remove(TEST_FILE)
        return True
    except:
        return False


_file_permissions = None


def ensure_writable_plotly_dir():
    # Cache permissions status
    global _file_permissions
    if _file_permissions is None:
        _file_permissions = _permissions()
    return _file_permissions



from ._z import ZValidator
from ._yshift import YshiftValidator
from ._yanchor import YanchorValidator
from ._y import YValidator
from ._xshift import XshiftValidator
from ._xanchor import XanchorValidator
from ._x import XValidator
from ._width import WidthValidator
from ._visible import VisibleValidator
from ._valign import ValignValidator
from ._textangle import TextangleValidator
from ._text import TextValidator
from ._templateitemname import TemplateitemnameValidator
from ._startstandoff import StartstandoffValidator
from ._startarrowsize import StartarrowsizeValidator
from ._startarrowhead import StartarrowheadValidator
from ._standoff import StandoffValidator
from ._showarrow import ShowarrowValidator
from ._opacity import OpacityValidator
from ._name import NameValidator
from ._hovertext import HovertextValidator
from ._hoverlabel import HoverlabelValidator
from ._height import HeightValidator
from ._font import FontValidator
from ._captureevents import CaptureeventsValidator
from ._borderwidth import BorderwidthValidator
from ._borderpad import BorderpadValidator
from ._bordercolor import BordercolorValidator
from ._bgcolor import BgcolorValidator
from ._ay import AyValidator
from ._ax import AxValidator
from ._arrowwidth import ArrowwidthValidator
from ._arrowsize import ArrowsizeValidator
from ._arrowside import ArrowsideValidator
from ._arrowhead import ArrowheadValidator
from ._arrowcolor import ArrowcolorValidator
from ._align import AlignValidator



import _plotly_utils.basevalidators


class RowValidator(_plotly_utils.basevalidators.IntegerValidator):

    def __init__(self, plotly_name='row', parent_name='pie.domain', **kwargs):
        super(RowValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop('edit_type', 'calc'),
            min=kwargs.pop('min', 0),
            role=kwargs.pop('role', 'info'),
            **kwargs
        )



import _plotly_utils.basevalidators


class NodeValidator(_plotly_utils.basevalidators.CompoundValidator):

    def __init__(self, plotly_name='node', parent_name='sankey', **kwargs):
        super(NodeValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop('data_class_str', 'Node'),
            data_docs=kwargs.pop(
                'data_docs', """
            color
                Sets the `node` color. It can be a single
                value, or an array for specifying color for
                each `node`. If `node.color` is omitted, then
                the default `Plotly` color palette will be
                cycled through to have a variety of colors.
                These defaults are not fully opaque, to allow
                some visibility of what is beneath the node.
            colorsrc
                Sets the source reference on plot.ly for  color
                .
            label
                The shown name of the node.
            labelsrc
                Sets the source reference on plot.ly for  label
                .
            line
                plotly.graph_objs.sankey.node.Line instance or
                dict with compatible properties
            pad
                Sets the padding (in px) between the `nodes`.
            thickness
                Sets the thickness (in px) of the `nodes`.
"""
            ),
            **kwargs
        )



from ._vertexnormalsepsilon import VertexnormalsepsilonValidator
from ._specular import SpecularValidator
from ._roughness import RoughnessValidator
from ._fresnel import FresnelValidator
from ._facenormalsepsilon import FacenormalsepsilonValidator
from ._diffuse import DiffuseValidator
from ._ambient import AmbientValidator



import _plotly_utils.basevalidators


class TextfontValidator(_plotly_utils.basevalidators.CompoundValidator):

    def __init__(
        self,
        plotly_name='textfont',
        parent_name='scatterpolar.selected',
        **kwargs
    ):
        super(TextfontValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop('data_class_str', 'Textfont'),
            data_docs=kwargs.pop(
                'data_docs', """
            color
                Sets the text font color of selected points.
"""
            ),
            **kwargs
        )



"""
Stand-alone module to provide information about whether optional deps exist.

"""
from __future__ import absolute_import

from importlib import import_module
import logging

logger = logging.getLogger(__name__)
_not_importable = set()


def get_module(name):
    """
    Return module or None. Absolute import is required.

    :param (str) name: Dot-separated module path. E.g., 'scipy.stats'.
    :raise: (ImportError) Only when exc_msg is defined.
    :return: (module|None) If import succeeds, the module will be returned.

    """
    if name not in _not_importable:
        try:
            return import_module(name)
        except ImportError:
            _not_importable.add(name)
        except Exception as e:
            _not_importable.add(name)
            msg = "Error importing optional module {}".format(name)
            logger.exception(msg)



import _plotly_utils.basevalidators


class GridValidator(_plotly_utils.basevalidators.CompoundValidator):

    def __init__(self, plotly_name='grid', parent_name='layout', **kwargs):
        super(GridValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop('data_class_str', 'Grid'),
            data_docs=kwargs.pop(
                'data_docs', """
            columns
                The number of columns in the grid. If you
                provide a 2D `subplots` array, the length of
                its longest row is used as the default. If you
                give an `xaxes` array, its length is used as
                the default. But it's also possible to have a
                different length, if you want to leave a row at
                the end for non-cartesian subplots.
            domain
                plotly.graph_objs.layout.grid.Domain instance
                or dict with compatible properties
            pattern
                If no `subplots`, `xaxes`, or `yaxes` are given
                but we do have `rows` and `columns`, we can
                generate defaults using consecutive axis IDs,
                in two ways: "coupled" gives one x axis per
                column and one y axis per row. "independent"
                uses a new xy pair for each cell, left-to-right
                across each row then iterating rows according
                to `roworder`.
            roworder
                Is the first row the top or the bottom? Note
                that columns are always enumerated from left to
                right.
            rows
                The number of rows in the grid. If you provide
                a 2D `subplots` array or a `yaxes` array, its
                length is used as the default. But it's also
                possible to have a different length, if you
                want to leave a row at the end for non-
                cartesian subplots.
            subplots
                Used for freeform grids, where some axes may be
                shared across subplots but others are not. Each
                entry should be a cartesian subplot id, like
                "xy" or "x3y2", or ** to leave that cell empty.
                You may reuse x axes within the same column,
                and y axes within the same row. Non-cartesian
                subplots and traces that support `domain` can
                place themselves in this grid separately using
                the `gridcell` attribute.
            xaxes
                Used with `yaxes` when the x and y axes are
                shared across columns and rows. Each entry
                should be an x axis id like "x", "x2", etc., or
                ** to not put an x axis in that column. Entries
                other than ** must be unique. Ignored if
                `subplots` is present. If missing but `yaxes`
                is present, will generate consecutive IDs.
            xgap
                Horizontal space between grid cells, expressed
                as a fraction of the total width available to
                one cell. Defaults to 0.1 for coupled-axes
                grids and 0.2 for independent grids.
            xside
                Sets where the x axis labels and titles go.
                "bottom" means the very bottom of the grid.
                *bottom plot* is the lowest plot that each x
                axis is used in. "top" and *top plot* are
                similar.
            yaxes
                Used with `yaxes` when the x and y axes are
                shared across columns and rows. Each entry
                should be an y axis id like "y", "y2", etc., or
                ** to not put a y axis in that row. Entries
                other than ** must be unique. Ignored if
                `subplots` is present. If missing but `xaxes`
                is present, will generate consecutive IDs.
            ygap
                Vertical space between grid cells, expressed as
                a fraction of the total height available to one
                cell. Defaults to 0.1 for coupled-axes grids
                and 0.3 for independent grids.
            yside
                Sets where the y axis labels and titles go.
                "left" means the very left edge of the grid.
                *left plot* is the leftmost plot that each y
                axis is used in. "right" and *right plot* are
                similar.
"""
            ),
            **kwargs
        )



import _plotly_utils.basevalidators


class BaseframeValidator(_plotly_utils.basevalidators.StringValidator):

    def __init__(self, plotly_name='baseframe', parent_name='frame', **kwargs):
        super(BaseframeValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            role=kwargs.pop('role', 'info'),
            **kwargs
        )



import plotly
from plotly.basedatatypes import BaseFigure
import plotly.graph_objs as go


def validate_coerce_fig_to_dict(fig, validate):
    if isinstance(fig, BaseFigure):
        fig_dict = fig.to_dict()
    elif isinstance(fig, dict):
        if validate:
            # This will raise an exception if fig is not a valid plotly figure
            fig_dict = plotly.graph_objs.Figure(fig).to_plotly_json()
        else:
            fig_dict = fig
    else:
        raise ValueError("""
The fig parameter must be a dict or Figure.
    Received value of type {typ}: {v}""".format(typ=type(fig), v=fig))
    return fig_dict


def validate_coerce_output_type(output_type):
    if output_type == 'Figure' or output_type == go.Figure:
        cls = go.Figure
    elif (output_type == 'FigureWidget' or
          (hasattr(go, 'FigureWidget') and output_type == go.FigureWidget)):
        cls = go.FigureWidget
    else:
        raise ValueError("""
Invalid output type: {output_type}
    Must be one of: 'Figure', 'FigureWidget'""")
    return cls


from ._zaxis import ZAxis
from plotly.graph_objs.layout.scene import zaxis
from ._yaxis import YAxis
from plotly.graph_objs.layout.scene import yaxis
from ._xaxis import XAxis
from plotly.graph_objs.layout.scene import xaxis
from ._domain import Domain
from ._camera import Camera
from plotly.graph_objs.layout.scene import camera
from ._aspectratio import Aspectratio
from ._annotation import Annotation
from plotly.graph_objs.layout.scene import annotation



from plotly.basedatatypes import BaseTraceHierarchyType
import copy


class XBins(BaseTraceHierarchyType):

    # end
    # ---
    @property
    def end(self):
        """
        Sets the end value for the x axis bins.
    
        The 'end' property accepts values of any type

        Returns
        -------
        Any
        """
        return self['end']

    @end.setter
    def end(self, val):
        self['end'] = val

    # size
    # ----
    @property
    def size(self):
        """
        Sets the step in-between value each x axis bin.
    
        The 'size' property accepts values of any type

        Returns
        -------
        Any
        """
        return self['size']

    @size.setter
    def size(self, val):
        self['size'] = val

    # start
    # -----
    @property
    def start(self):
        """
        Sets the starting value for the x axis bins.
    
        The 'start' property accepts values of any type

        Returns
        -------
        Any
        """
        return self['start']

    @start.setter
    def start(self, val):
        self['start'] = val

    # property parent name
    # --------------------
    @property
    def _parent_path_str(self):
        return 'histogram2dcontour'

    # Self properties description
    # ---------------------------
    @property
    def _prop_descriptions(self):
        return """\
        end
            Sets the end value for the x axis bins.
        size
            Sets the step in-between value each x axis bin.
        start
            Sets the starting value for the x axis bins.
        """

    def __init__(self, arg=None, end=None, size=None, start=None, **kwargs):
        """
        Construct a new XBins object
        
        Parameters
        ----------
        arg
            dict of properties compatible with this constructor or
            an instance of
            plotly.graph_objs.histogram2dcontour.XBins
        end
            Sets the end value for the x axis bins.
        size
            Sets the step in-between value each x axis bin.
        start
            Sets the starting value for the x axis bins.

        Returns
        -------
        XBins
        """
        super(XBins, self).__init__('xbins')

        # Validate arg
        # ------------
        if arg is None:
            arg = {}
        elif isinstance(arg, self.__class__):
            arg = arg.to_plotly_json()
        elif isinstance(arg, dict):
            arg = copy.copy(arg)
        else:
            raise ValueError(
                """\
The first argument to the plotly.graph_objs.histogram2dcontour.XBins 
constructor must be a dict or 
an instance of plotly.graph_objs.histogram2dcontour.XBins"""
            )

        # Handle skip_invalid
        # -------------------
        self._skip_invalid = kwargs.pop('skip_invalid', False)

        # Import validators
        # -----------------
        from plotly.validators.histogram2dcontour import (xbins as v_xbins)

        # Initialize validators
        # ---------------------
        self._validators['end'] = v_xbins.EndValidator()
        self._validators['size'] = v_xbins.SizeValidator()
        self._validators['start'] = v_xbins.StartValidator()

        # Populate data dict with properties
        # ----------------------------------
        _v = arg.pop('end', None)
        self['end'] = end if end is not None else _v
        _v = arg.pop('size', None)
        self['size'] = size if size is not None else _v
        _v = arg.pop('start', None)
        self['start'] = start if start is not None else _v

        # Process unknown kwargs
        # ----------------------
        self._process_kwargs(**dict(arg, **kwargs))

        # Reset skip_invalid
        # ------------------
        self._skip_invalid = False



import _plotly_utils.basevalidators


class ResolutionValidator(_plotly_utils.basevalidators.EnumeratedValidator):

    def __init__(
        self, plotly_name='resolution', parent_name='layout.geo', **kwargs
    ):
        super(ResolutionValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            coerce_number=kwargs.pop('coerce_number', True),
            edit_type=kwargs.pop('edit_type', 'plot'),
            role=kwargs.pop('role', 'info'),
            values=kwargs.pop('values', [110, 50]),
            **kwargs
        )



from plotly.basedatatypes import BaseTraceHierarchyType
import copy


class Axis(BaseTraceHierarchyType):

    # type
    # ----
    @property
    def type(self):
        """
        Sets the axis type for this dimension's generated x and y axes.
        Note that the axis `type` values set in layout take precedence
        over this attribute.
    
        The 'type' property is an enumeration that may be specified as:
          - One of the following enumeration values:
                ['linear', 'log', 'date', 'category']

        Returns
        -------
        Any
        """
        return self['type']

    @type.setter
    def type(self, val):
        self['type'] = val

    # property parent name
    # --------------------
    @property
    def _parent_path_str(self):
        return 'splom.dimension'

    # Self properties description
    # ---------------------------
    @property
    def _prop_descriptions(self):
        return """\
        type
            Sets the axis type for this dimension's generated x and
            y axes. Note that the axis `type` values set in layout
            take precedence over this attribute.
        """

    def __init__(self, arg=None, type=None, **kwargs):
        """
        Construct a new Axis object
        
        Parameters
        ----------
        arg
            dict of properties compatible with this constructor or
            an instance of plotly.graph_objs.splom.dimension.Axis
        type
            Sets the axis type for this dimension's generated x and
            y axes. Note that the axis `type` values set in layout
            take precedence over this attribute.

        Returns
        -------
        Axis
        """
        super(Axis, self).__init__('axis')

        # Validate arg
        # ------------
        if arg is None:
            arg = {}
        elif isinstance(arg, self.__class__):
            arg = arg.to_plotly_json()
        elif isinstance(arg, dict):
            arg = copy.copy(arg)
        else:
            raise ValueError(
                """\
The first argument to the plotly.graph_objs.splom.dimension.Axis 
constructor must be a dict or 
an instance of plotly.graph_objs.splom.dimension.Axis"""
            )

        # Handle skip_invalid
        # -------------------
        self._skip_invalid = kwargs.pop('skip_invalid', False)

        # Import validators
        # -----------------
        from plotly.validators.splom.dimension import (axis as v_axis)

        # Initialize validators
        # ---------------------
        self._validators['type'] = v_axis.TypeValidator()

        # Populate data dict with properties
        # ----------------------------------
        _v = arg.pop('type', None)
        self['type'] = type if type is not None else _v

        # Process unknown kwargs
        # ----------------------
        self._process_kwargs(**dict(arg, **kwargs))

        # Reset skip_invalid
        # ------------------
        self._skip_invalid = False



import _plotly_utils.basevalidators


class LineValidator(_plotly_utils.basevalidators.CompoundValidator):

    def __init__(
        self, plotly_name='line', parent_name='scatterpolargl', **kwargs
    ):
        super(LineValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop('data_class_str', 'Line'),
            data_docs=kwargs.pop(
                'data_docs', """
            color
                Sets the line color.
            dash
                Sets the style of the lines.
            width
                Sets the line width (in px).
"""
            ),
            **kwargs
        )



import _plotly_utils.basevalidators


class TokenValidator(_plotly_utils.basevalidators.StringValidator):

    def __init__(
        self, plotly_name='token', parent_name='mesh3d.stream', **kwargs
    ):
        super(TokenValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop('edit_type', 'calc'),
            no_blank=kwargs.pop('no_blank', True),
            role=kwargs.pop('role', 'info'),
            strict=kwargs.pop('strict', True),
            **kwargs
        )



import _plotly_utils.basevalidators


class FramesValidator(_plotly_utils.basevalidators.CompoundArrayValidator):

    def __init__(self, plotly_name='frames', parent_name='', **kwargs):
        super(FramesValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str=kwargs.pop('data_class_str', 'Frame'),
            data_docs=kwargs.pop(
                'data_docs', """
            baseframe
                The name of the frame into which this frame's
                properties are merged before applying. This is
                used to unify properties and avoid needing to
                specify the same values for the same properties
                in multiple frames.
            data
                A list of traces this frame modifies. The
                format is identical to the normal trace
                definition.
            group
                An identifier that specifies the group to which
                the frame belongs, used by animate to select a
                subset of frames.
            layout
                Layout properties which this frame modifies.
                The format is identical to the normal layout
                definition.
            name
                A label by which to identify the frame
            traces
                A list of trace indices that identify the
                respective traces in the data attribute
"""
            ),
            **kwargs
        )



import _plotly_utils.basevalidators


class XcalendarValidator(_plotly_utils.basevalidators.EnumeratedValidator):

    def __init__(
        self,
        plotly_name='xcalendar',
        parent_name='histogram2dcontour',
        **kwargs
    ):
        super(XcalendarValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type=kwargs.pop('edit_type', 'calc'),
            role=kwargs.pop('role', 'info'),
            values=kwargs.pop(
                'values', [
                    'gregorian', 'chinese', 'coptic', 'discworld', 'ethiopian',
                    'hebrew', 'islamic', 'julian', 'mayan', 'nanakshahi',
                    'nepali', 'persian', 'jalali', 'taiwan', 'thai',
                    'ummalqura'
                ]
            ),
            **kwargs
        )



import _plotly_utils.basevalidators


class DataValidator(_plotly_utils.basevalidators.BaseDataValidator):

    def __init__(self, plotly_name='data', parent_name='', **kwargs):

        super(DataValidator, self).__init__(
            class_strs_map={
                'area': 'Area',
                'bar': 'Bar',
                'barpolar': 'Barpolar',
                'box': 'Box',
                'candlestick': 'Candlestick',
                'carpet': 'Carpet',
                'choropleth': 'Choropleth',
                'cone': 'Cone',
                'contour': 'Contour',
                'contourcarpet': 'Contourcarpet',
                'heatmap': 'Heatmap',
                'heatmapgl': 'Heatmapgl',
                'histogram': 'Histogram',
                'histogram2d': 'Histogram2d',
                'histogram2dcontour': 'Histogram2dContour',
                'mesh3d': 'Mesh3d',
                'ohlc': 'Ohlc',
                'parcoords': 'Parcoords',
                'pie': 'Pie',
                'pointcloud': 'Pointcloud',
                'sankey': 'Sankey',
                'scatter': 'Scatter',
                'scatter3d': 'Scatter3d',
                'scattercarpet': 'Scattercarpet',
                'scattergeo': 'Scattergeo',
                'scattergl': 'Scattergl',
                'scattermapbox': 'Scattermapbox',
                'scatterpolar': 'Scatterpolar',
                'scatterpolargl': 'Scatterpolargl',
                'scatterternary': 'Scatterternary',
                'splom': 'Splom',
                'streamtube': 'Streamtube',
                'surface': 'Surface',
                'table': 'Table',
                'violin': 'Violin',
            },
            plotly_name=plotly_name,
            parent_name=parent_name,
            **kwargs
        )



from plotly.basedatatypes import BaseLayoutHierarchyType
import copy


class Line(BaseLayoutHierarchyType):

    # width
    # -----
    @property
    def width(self):
        """
        Sets the line width. Has an effect only when `type` is set to
        "line".
    
        The 'width' property is a number and may be specified as:
          - An int or float

        Returns
        -------
        int|float
        """
        return self['width']

    @width.setter
    def width(self, val):
        self['width'] = val

    # property parent name
    # --------------------
    @property
    def _parent_path_str(self):
        return 'layout.mapbox.layer'

    # Self properties description
    # ---------------------------
    @property
    def _prop_descriptions(self):
        return """\
        width
            Sets the line width. Has an effect only when `type` is
            set to "line".
        """

    def __init__(self, arg=None, width=None, **kwargs):
        """
        Construct a new Line object
        
        Parameters
        ----------
        arg
            dict of properties compatible with this constructor or
            an instance of
            plotly.graph_objs.layout.mapbox.layer.Line
        width
            Sets the line width. Has an effect only when `type` is
            set to "line".

        Returns
        -------
        Line
        """
        super(Line, self).__init__('line')

        # Validate arg
        # ------------
        if arg is None:
            arg = {}
        elif isinstance(arg, self.__class__):
            arg = arg.to_plotly_json()
        elif isinstance(arg, dict):
            arg = copy.copy(arg)
        else:
            raise ValueError(
                """\
The first argument to the plotly.graph_objs.layout.mapbox.layer.Line 
constructor must be a dict or 
an instance of plotly.graph_objs.layout.mapbox.layer.Line"""
            )

        # Handle skip_invalid
        # -------------------
        self._skip_invalid = kwargs.pop('skip_invalid', False)

        # Import validators
        # -----------------
        from plotly.validators.layout.mapbox.layer import (line as v_line)

        # Initialize validators
        # ---------------------
        self._validators['width'] = v_line.WidthValidator()

        # Populate data dict with properties
        # ----------------------------------
        _v = arg.pop('width', None)
        self['width'] = width if width is not None else _v

        # Process unknown kwargs
        # ----------------------
        self._process_kwargs(**dict(arg, **kwargs))

        # Reset skip_invalid
        # ------------------
        self._skip_invalid = False
